#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "list.h"
#include "Painter.h"
#include "Level.h"

Level aLevel;   

Level LoadLevel(char* FileName)
{
 int i;
 
 Level theLevel; 
 theLevel.Motion = false;
 theLevel.Active = ISRED;
 theLevel.Size = 10;
 freeList(PlayList);
 PlayList = CreateNewTList();
 FILE* NewLevel = fopen(FileName,"r");		//This opens a File so we can read it in line by line.  

 if (!NewLevel)
 {
  printf("%s not found.\r\n", FileName);
  exit(1);
 }

 int LineCounter = 0;		//This will make sure we only count 10 lines into the Stage array.  So if there is any
 char nextLevelLine[27];	// more level data, it gets ignored.
 
 while (fgets(nextLevelLine,27,NewLevel)!=NULL && LineCounter < theLevel.Size)
 { 
  for(i = 0; i < theLevel.Size; i++)
  {
   if(nextLevelLine[i] == '0') theLevel.Walls[i][LineCounter] = MakeWallTile(BNONE);
   else if(nextLevelLine[i] == '1') theLevel.Walls[i][LineCounter] = MakeWallTile(BNORTH);
   else if(nextLevelLine[i] == '2') theLevel.Walls[i][LineCounter] = MakeWallTile(BEAST);
   else if(nextLevelLine[i] == '3') theLevel.Walls[i][LineCounter] = MakeWallTile(BSOUTH);
   else if(nextLevelLine[i] == '4') theLevel.Walls[i][LineCounter] = MakeWallTile(BWEST);
   else if(nextLevelLine[i] == '5') theLevel.Walls[i][LineCounter] = MakeWallTile(BNORTHEAST);
   else if(nextLevelLine[i] == '6') theLevel.Walls[i][LineCounter] = MakeWallTile(BNORTHSOUTH);
   else if(nextLevelLine[i] == '7') theLevel.Walls[i][LineCounter] = MakeWallTile(BNORTHWEST);
   else if(nextLevelLine[i] == '8') theLevel.Walls[i][LineCounter] = MakeWallTile(BEASTSOUTH);
   else if(nextLevelLine[i] == '9') theLevel.Walls[i][LineCounter] = MakeWallTile(BEASTWEST);
   else if(nextLevelLine[i] == 'A') theLevel.Walls[i][LineCounter] = MakeWallTile(BSOUTHWEST);
   else if(nextLevelLine[i] == 'B') theLevel.Walls[i][LineCounter] = MakeWallTile(BNORTHEASTSOUTH);
   else if(nextLevelLine[i] == 'C') theLevel.Walls[i][LineCounter] = MakeWallTile(BEASTSOUTHWEST);
   else if(nextLevelLine[i] == 'D') theLevel.Walls[i][LineCounter] = MakeWallTile(BSOUTHWESTNORTH);
   else if(nextLevelLine[i] == 'E') theLevel.Walls[i][LineCounter] = MakeWallTile(BWESTNORTHEAST);
   else if(nextLevelLine[i] == 'F') theLevel.Walls[i][LineCounter] = MakeWallTile(BALL);
  }       
  LineCounter++;
  printf("%s", nextLevelLine);	
 }
 do
 {
  if(nextLevelLine[0] == 'P')
  {
    int x, y, colour, move, point;
    char* Name = (char*)malloc(512);
    char Ind;
    sscanf(nextLevelLine, "%c %d %d %d %d %d %s", &Ind, &x, &y, &colour, &move, &point, Name);
    printf("%c %d %d %d %d %d %s\r\n", Ind, x, y, colour, move, point, Name);
    
    ColourType Col;
    if(colour == 0) Col = ISGREY;
    else if(colour == 1) Col = ISRED;
    else Col = ISYELLOW;
    
    PointAt PAt;
    
    if(point == 0) PAt = PNONE;
    else if(point == 1) PAt = PNORTH;
    else if(point == 2) PAt = PEAST;
    else if(point == 3) PAt = PSOUTH;
    else PAt = PWEST;
    
    PlayList = addNode(PlayList, MakeStandardTile(Name, Col, move, PAt, x, y));
  }
  else if(nextLevelLine[0] == 'W')
  {
   int x, y, colour, dirty, hole;
   char Ind;
   sscanf(nextLevelLine, "%c %d %d %d %d %d", &Ind, &x, &y, &colour, &dirty, &hole);
   printf("%c %d %d %d %d %d\r\n", Ind, x, y, colour, dirty, hole);   
   
   ColourType Col;
   if(colour == 0) Col = ISGREY;
   else if(colour == 1) Col = ISRED;
   else if(colour == 2) Col = ISYELLOW;
   
   theLevel.Walls[x][y].Colour = Col;
   theLevel.Walls[x][y].Dirty = dirty;
   theLevel.Walls[x][y].Hole = hole;
  }
 }while(fgets(nextLevelLine,27,NewLevel) != NULL);
 
 fclose(NewLevel);
 return theLevel;
}

void PrintLevel(Level theLevel)
{
 for(int i = 0; i < theLevel.Size; i++)
   for(int j = 0; j < theLevel.Size; j++)
     PaintWall(theLevel.Walls[i][j], i+2, j+2);
 PlayList = ResetList(PlayList);
 while(PlayList.Current != NULL)
 {
  PaintCharacter(PlayList.Current->theTile.Name, 
                 PlayList.Current->theTile.X+2, PlayList.Current->theTile.Y+2, 
                 PlayList.Current->theTile.Colour, PlayList.Current->theTile.PointDirection);    
  PlayList = nextNode(PlayList);
 }
}

void MoveWest()
{
 PlayList = ResetList(PlayList);
 while(PlayList.Current != NULL)
 {
  PlayList.Current->theTile = SetDirectionByPlayer(PlayList.Current->theTile, WEST);  
  PlayList = nextNode(PlayList);
 }     

 aLevel.Motion = true;
}

void MoveEast()
{
 PlayList = ResetList(PlayList);
 while(PlayList.Current != NULL)
 {
  PlayList.Current->theTile = SetDirectionByPlayer(PlayList.Current->theTile, EAST);  
  PlayList = nextNode(PlayList);
 }    
 aLevel.Motion = true;     
}

void MoveNorth()
{
 PlayList = ResetList(PlayList);
 while(PlayList.Current != NULL)
 {
  PlayList.Current->theTile = SetDirectionByPlayer(PlayList.Current->theTile, NORTH);  
  PlayList = nextNode(PlayList);
 }    
 aLevel.Motion = true;     
}

void MoveSouth()
{
 PlayList = ResetList(PlayList);
 while(PlayList.Current != NULL)
 {
  PlayList.Current->theTile = SetDirectionByPlayer(PlayList.Current->theTile, SOUTH);  
  PlayList = nextNode(PlayList);
 }    
 aLevel.Motion = true;     
}

PlayTile TestWalls(Level theLevel, PlayTile theTile)
{
 if(theTile.Moving == STOPPED || theTile.Moving == PUSH) return theTile;

 if(theLevel.Walls[theTile.X][theTile.Y].Hole)
 {
  theTile.RemoveMe = true;
  return theTile;
 }

 if(theLevel.Walls[theTile.X][theTile.Y].Dirty && !theTile.Dirty) 
 {
   if(theTile.InTheWay) theTile.Moving = STOPPED;
   theTile.Dirty = true;
   return theTile;
 }
 else if(theTile.Dirty) theTile.Dirty = false;
 
 if(theLevel.Walls[theTile.X][theTile.Y].Colour != ISGREY &&
         theLevel.Walls[theTile.X][theTile.Y].Colour != theLevel.Active){} 
 else if(theLevel.Walls[theTile.X][theTile.Y].Blocks[theTile.Moving-1]) theTile.Moving = STOPPED;
 
 return theTile;        
}

void DisplayCallback()
{
  // clear the color buffer
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  // set up camera
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( 45, 1.0, 1.0, 6000.0 );

  glMatrixMode( GL_MODELVIEW );
  GLdouble modelview[16];
  glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
  glLoadIdentity();
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	
  glutSwapBuffers();
}

void KeyboardDown(unsigned char c, int x, int y) 
{
  if(!aLevel.Motion)   
  {
   switch (c) 
   {
    case 27:
      exit (0);
      break;
    case 'a':
      MoveWest();
      break;      
    case 's':
      MoveSouth();
      break;    
    case 'w':
      MoveNorth();
      break;    
    case 'd':
      MoveEast();
      break;                      
   }
  }
}

void onIdle()
{
 if(PlayList.GameOver) exit(1);
 if(PlayList.Winner) exit(1);
 if(aLevel.Motion)
 {                   
  PlayList = ResetList(PlayList);
  bool Stop = true;

  PlayList = ResetList(PlayList);
  while(PlayList.Current != NULL) 
  {
   if(PlayList.Current->theTile.MoveType == BLOCK &&  
      PlayList.Current->theTile.Moving == STOPPED) PlayList.Current->theTile.Moving = PUSH;
   PlayList.Current->theTile = TestWalls(aLevel, PlayList.Current->theTile);
   PlayList = nextNode(PlayList);
  }    
  TNode* Active = PlayList.Head;  
  bool Swap = false;

  while(Active != NULL)
  {                             
   if(Swap) Active->theTile = TestWalls(aLevel, Active->theTile);
   if(Active->theTile.Moving != STOPPED &&
      Active->theTile.Moving != PUSH) Stop = false;
   Swap = false;
   bool AlreadyMoved = false;
   TNode* Reactive = PlayList.Head;
   while(Reactive != NULL)
   {
    int Interact;
    if(Active == Reactive) Interact = 0;
    else Interact = SeeInteraction(Active->theTile, Reactive->theTile, aLevel.Active);
    if(Interact == 2)
    {         
     PlayList = SwapNodes(PlayList, Active, Reactive);            

     Reactive = NULL;
     Swap = true;          
    }
    else if(Interact == 1)
    {
     if(Reactive->theTile.Colour == SETRED && Active->theTile.InTheWay) aLevel.Active = ISRED;
     else if(Reactive->theTile.Colour == SETYELLOW && Active->theTile.InTheWay) aLevel.Active = ISYELLOW;

     int result = RemoveTile(Active->theTile, Reactive->theTile, aLevel.Active);
     if(result == 0) 
     {
      int direction = DetermineNewDirection(Active->theTile, Reactive->theTile, aLevel.Active);
      if(direction == 0) 
      {
        bool position = Takeover(Active->theTile, Reactive->theTile, aLevel.Active);
        if(!position) { Active->theTile.Moving = STOPPED; Reactive->theTile.Moving = STOPPED; }
      }
      else if(direction == 1) 
        Reactive->theTile.Moving = (Direction) Active->theTile.PointDirection;
      else if(direction == 2) 
      {
        bool position = Takeover(Active->theTile, Reactive->theTile, aLevel.Active);
        if(!position) { Active->theTile.Moving = STOPPED; Reactive->theTile.Moving = STOPPED; }
        Active->theTile.NewMoving = (PointAt) Reactive->theTile.PointDirection;
      }
      else if(direction == 3) 
      {
       Reactive->theTile.Moving = Active->theTile.Moving; 
       PlayList = SwapNodes(PlayList, Active, Reactive); 
       Reactive = NULL;
       Swap = true;   
      }
     }
     else if(result == 1) 
     {
      Active->theTile.RemoveMe = true;
      if(Active->theTile.Collect == COLLECTABLE && Reactive->theTile.Collect == COLLECTOR)
        Active->theTile.Collected = true;
     }
     else if(result == 2) 
     {
       Reactive->theTile.RemoveMe = true;
       if(Reactive->theTile.Collect == COLLECTABLE && Active->theTile.Collect == COLLECTOR)
        Reactive->theTile.Collected = true;       
     }
     else if(result == 3) 
     { 
       Active->theTile.RemoveMe = true; 
       Reactive->theTile.RemoveMe = true; 
       if(Reactive->theTile.Collect == COLLECTABLE && Active->theTile.Collect == COLLECTOR)
         Reactive->theTile.Collected = true;  
       if(Active->theTile.Collect == COLLECTABLE && Reactive->theTile.Collect == COLLECTOR)
         Active->theTile.Collected = true;              
     }
     if(!Swap) Reactive = Reactive->next;
    } else if(Interact == 0) Reactive = Reactive->next;
   } 
   if(!Swap)
   {
     if(Active->theTile.Moving == NORTH) Active->theTile.Y--;
     else if(Active->theTile.Moving == WEST) Active->theTile.X--;
     else if(Active->theTile.Moving == EAST) Active->theTile.X++;
     else if(Active->theTile.Moving == SOUTH) Active->theTile.Y++; 
     if(Active->theTile.X < 0 || Active->theTile.X >= aLevel.Size || Active->theTile.Y < 0 || Active->theTile.Y >= aLevel.Size) Active->theTile.RemoveMe = true;
     
     if(Active->theTile.NewMoving > PNONE) 
     {
      Active->theTile.Moving = (Direction) Active->theTile.NewMoving;
      Active->theTile.NewMoving = PNONE;
     }      
     Active = Active->next;
   }
  }

  PlayList = removeNodes(PlayList);
  if(Stop) aLevel.Motion = false;
  PrintLevel(aLevel);                 
 }
}

int main(int argc, char **argv)
{
 glutInit( &argc, argv );    
 if(argc <= 1) exit(1);
 aLevel = LoadLevel(argv[1]);
 ClearScreen();  
 PrintLevel(aLevel);
 glutInitDisplayMode( GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE );
 glutInitWindowSize( 20, 20 );
 glutCreateWindow("Project 3, KourikuGL");
  // register display callback
 glutDisplayFunc( DisplayCallback );
 glutKeyboardFunc( KeyboardDown );
 glutIdleFunc(onIdle);
 glViewport( 0, 0, 700, 700 );
 glEnable( GL_DEPTH_TEST );
 
 glutMainLoop();
  
 return 0;       // never reached 
}
