/*
Kouriku Snow
Michael Vendittelli 
*/

#ifndef gf_included_PlayTile_h
#define gf_included_PlayTile_h
/*
-Name      : A string that will tell us the name of the Tile.  This is mostly for some basic logic and painting.
-Moving    : Is used to tell us in which direction the tile is moving in, if it has stopped, or if it must be pushed.
-Liveable  : Is the tile alive?
-Moveable  : Will the tile move when key is pressed?
-GivesDirections: The tile will give a direction to another if asked.
-ForceDirections: The tile will force it's pointat direction on another.
-PointAt   : This is set to NONE or N, E, S, W.  If the PointAt is set, then it will give that direction to a tile it 	     touches.  Otherwise it will give the direction it is moving in (N, E, S, W).
-Colour    : This tells us the tile's colour.
-Alignment : Tells us if the tile is a Hero, an Enemy, or a Neutral.  Heroes and Enemies kill eachother.
-Murderer  : Will the tile kill *any* Liveable it touches?
-Suicidal  : Will the tile die when it kills?
-Important : If an important tile dies, the level is unsuccessful.
-Collect   : Set to either NO, COLLECTWAIT, COLLECTMEWAIT, COLLECTOR, or COLLECTABLE.  Collectors will collect collectables.  When all collectables are removed, collectors become collectables and COLLECTWAITs become Collectors.  COLLECTMEWAITs become Collectables.  If Collectors collect all collectables a second time, the level is completed successfully.
-DirType   : Enum type that tells us if it accepts keypress moves like Pid, like Flames, or like Blocks.
-Allowed  : These four fields are boolean flags, they tell us what directions are allowed to be taken by this object
	     when it is ordered (not by keypresses, but another tile).  If a direction is ordered upon this tile and it isn't 	     allowed, we ignore the order.
-InTheWay  : Does it take up the rest of this tilespace, or will it be pushed down?
-CoordX    : 
-CoordY    : The current 2D coords of the object.
*/

#include "types.h"

struct aPlayTile
{
 char* Name;

 Direction Moving;
 bool Moveable;
 bool GivesDirections;
 bool ForceDirections;
 PointAt PointDirection;
 PointAt NewMoving;
 DirType MoveType;
 bool Allowed[4]; 
 
 int X;
 int Y;
 
 ColourType Colour;
 AlignType Alignment;
 CollectType Collect;
 bool Important;
 bool Murderer;
 bool Suicidal;
 bool Liveable;
 bool InTheWay;
 bool RemoveMe;
 bool Collected;
 bool Dirty;
};

typedef struct aPlayTile PlayTile;

PlayTile MakePlayTile(char* pName, Direction pMoving, bool pMoveable, bool pGivesDirections, 
                   bool pForceDirections, PointAt pPointDirection, DirType pMoveType, 
                   bool pAllowed[4], int pX, int pY, ColourType pColour, AlignType pAlignment,
                   bool pMurderer, bool pSuicidal, bool pLiveable, bool pInTheWay, 
                   bool pImportant, CollectType pCollect);

PlayTile MakeStandardTile(char* pName, ColourType pColour, bool pMoveable, PointAt pPointDirection, int X, int Y);
PlayTile SetDirectionByPlayer(PlayTile theTile, Direction NewDir);
bool Takeover(PlayTile TileA, PlayTile TileB, ColourType Active);
//true if TileA moves to the position of TileB.
//false if it should stay put.

int RemoveTile(PlayTile TileA, PlayTile TileB, ColourType Active);
//Tile A is hitting Tile B.
//Returns 0 if both are still present.  1 if removing TileA.  2 if removing TileB.  3 if
//removing both.

int DetermineNewDirection(PlayTile TileA, PlayTile TileB, ColourType Active);
//returns 0 if Tile A nor B are changing directions.  returns 1 if Tile A is forcing a change on
// TileB.  2 if tile B is forcing change on TileA.  3 if Tile A is giving directions to B.  

int SeeInteraction(PlayTile TileA, PlayTile TileB, ColourType Active);
//returns 0 if there is no possible interaction between these two tiles.
//returns 1 if there is an interaction.
//returns 2 if there could be an interaction, but TileB may be resolved beforehand.
#endif
